//=========================================================================
//  TGraphWindow 1.0
//
//  M.E. Burkard    Copyright (C) 1998
//
//  OWL 5.0 / BC++ 5.01A code
//  A TWindow-derived class which draws a graph based on data passed.
//
//  THIS PROGRAM MAY BE FREELY USED AND MODIFIED
//  Mark Burkard is not liable from any results of the use or misuse
//  of this program.
//=========================================================================


#include <owl\chooseco.h>
#include <owl\edit.h>
#include <owl\updown.h>
#include <owl\printer.h>
#include <owl\clipboar.h>
#include <math.h>
#include <string.h>
#include <classlib\arrays.h>
#include <owl\window.h>
#include <owl\framewin.h>
#include <owl\dc.h>
#include <owl\applicat.h>

#pragma hdrstop

#include "thickdia.cpp"
#include "TWinPrint.cpp"

#include "TGraphWindow.h"


//-----------SUPPORTING FUNCTIONS-----------------------------------------

double abs(double i)
{
	if (i < 0) return -i;
   return i;
}

double round(double i)
{
	return ceil(i-0.5);
}


void split(double val, double& mant, int& exp)
// breaks val into mantissa and exponent

{
   int count;
   if (val > 10) {
		for (count = 0; val > 10.0; count++) {
   		val = val/10.0;
	   }
   }
   else {
   	for (count = 0; val < 1.0; count--) {
      	val = val*10.0;
   	}
   }
   mant = val;     // between 1 and 10
   exp = count;    // power of 10
}

double splice(double mant, int exp)
// joins val and mantissa to return double
{
	double v = mant;
   int count = exp;
   if (count > 0) {
   	while (count) {
			v = v*10.0;
      	count--;
      }
   }
   if (count < 0) {
   	while (count) {
      	v = v/10.0;
         count++;
      }
   }
   return v;
}



double getnearest(double v)
	// returns the nearest number that has only one non-zero digit
{
	int count;
   double mant;
	split(v, mant, count);
   mant = round(mant);
	v = splice(mant, count);
   return v;
}

int digits(double val)
{
	if (val < 100) return 2;
   if (val < 1000) return 3;
   if (val < 10000) return 4;
   return 5;
}


//==========================================================================
// TGraphWindow
//
// Graphs up to 15 y variables against an x variable
//==========================================================================

DEFINE_RESPONSE_TABLE1(TGraphWindow, TWindow)
	EV_WM_SIZE,
  	EV_COMMAND(PAGE_SETUP, CmPageSetup),
   EV_COMMAND(EXIT, CmExit),
   EV_COMMAND(PRINT, CmPrint),
   EV_COMMAND(COPY, CmCopy),
   EV_COMMAND(DISPLAY - 3, DispTitle),

   EV_COMMAND_ENABLE (DISPLAY - 3, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 2, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 3, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 4, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 5, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 6, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 7, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 8, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 9, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 10, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 12, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 13, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 14, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 15, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 16, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 17, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 18, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 19, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 20, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 22, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 23, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 24, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 25, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 26, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 27, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 28, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 29, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 30, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 32, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 33, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 34, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 35, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 36, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 37, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 38, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 39, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 40, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 42, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 43, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 44, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 45, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 46, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 47, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 48, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 49, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 50, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 52, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 53, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 54, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 55, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 56, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 57, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 58, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 59, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 60, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 62, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 63, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 64, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 65, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 66, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 67, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 68, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 69, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 70, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 72, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 73, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 74, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 75, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 76, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 77, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 78, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 79, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 80, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 82, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 83, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 84, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 85, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 86, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 87, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 88, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 89, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 90, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 92, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 93, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 94, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 95, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 96, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 97, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 98, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 99, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 100, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 102, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 103, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 104, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 105, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 106, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 107, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 108, CommandEnabler),
   EV_COMMAND_ENABLE (DISPLAY + 109, CommandEnabler),

   EV_COMMAND(DISPLAY - 3, DispTitle),
   EV_COMMAND_AND_ID(DISPLAY, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 2, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 3, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 4, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 5, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 6, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 7, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 8, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 9, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 10, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 12, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 13, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 14, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 15, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 16, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 17, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 18, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 19, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 20, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 22, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 23, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 24, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 25, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 26, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 27, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 28, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 29, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 30, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 32, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 33, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 34, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 35, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 36, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 37, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 38, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 39, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 40, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 42, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 43, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 44, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 45, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 46, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 47, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 48, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 49, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 50, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 52, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 53, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 54, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 55, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 56, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 57, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 58, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 59, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 60, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 62, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 63, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 64, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 65, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 66, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 67, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 68, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 69, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 70, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 72, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 73, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 74, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 75, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 76, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 77, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 78, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 79, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 80, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 82, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 83, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 84, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 85, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 86, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 87, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 88, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 89, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 90, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 92, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 93, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 94, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 95, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 96, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 97, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 98, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 99, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 100, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 102, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 103, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 104, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 105, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 106, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 107, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 108, InvDisplay),
   EV_COMMAND_AND_ID(DISPLAY + 109, InvDisplay),

   EV_COMMAND_AND_ID(COLOR,     SetColor),
   EV_COMMAND_AND_ID(COLOR + 2, SetColor),
   EV_COMMAND_AND_ID(COLOR + 3, SetColor),
   EV_COMMAND_AND_ID(COLOR + 4, SetColor),
   EV_COMMAND_AND_ID(COLOR + 5, SetColor),
   EV_COMMAND_AND_ID(COLOR + 6, SetColor),
   EV_COMMAND_AND_ID(COLOR + 7, SetColor),
   EV_COMMAND_AND_ID(COLOR + 8, SetColor),
   EV_COMMAND_AND_ID(COLOR + 9, SetColor),
   EV_COMMAND_AND_ID(COLOR + 10, SetColor),
   EV_COMMAND_AND_ID(COLOR + 12, SetColor),
   EV_COMMAND_AND_ID(COLOR + 13, SetColor),
   EV_COMMAND_AND_ID(COLOR + 14, SetColor),
   EV_COMMAND_AND_ID(COLOR + 15, SetColor),
   EV_COMMAND_AND_ID(COLOR + 16, SetColor),
   EV_COMMAND_AND_ID(COLOR + 17, SetColor),
   EV_COMMAND_AND_ID(COLOR + 18, SetColor),
   EV_COMMAND_AND_ID(COLOR + 19, SetColor),
   EV_COMMAND_AND_ID(COLOR + 20, SetColor),
   EV_COMMAND_AND_ID(COLOR + 22, SetColor),
   EV_COMMAND_AND_ID(COLOR + 23, SetColor),
   EV_COMMAND_AND_ID(COLOR + 24, SetColor),
   EV_COMMAND_AND_ID(COLOR + 25, SetColor),
   EV_COMMAND_AND_ID(COLOR + 26, SetColor),
   EV_COMMAND_AND_ID(COLOR + 27, SetColor),
   EV_COMMAND_AND_ID(COLOR + 28, SetColor),
   EV_COMMAND_AND_ID(COLOR + 29, SetColor),
   EV_COMMAND_AND_ID(COLOR + 30, SetColor),
   EV_COMMAND_AND_ID(COLOR + 32, SetColor),
   EV_COMMAND_AND_ID(COLOR + 33, SetColor),
   EV_COMMAND_AND_ID(COLOR + 34, SetColor),
   EV_COMMAND_AND_ID(COLOR + 35, SetColor),
   EV_COMMAND_AND_ID(COLOR + 36, SetColor),
   EV_COMMAND_AND_ID(COLOR + 37, SetColor),
   EV_COMMAND_AND_ID(COLOR + 38, SetColor),
   EV_COMMAND_AND_ID(COLOR + 39, SetColor),
   EV_COMMAND_AND_ID(COLOR + 40, SetColor),
   EV_COMMAND_AND_ID(COLOR + 42, SetColor),
   EV_COMMAND_AND_ID(COLOR + 43, SetColor),
   EV_COMMAND_AND_ID(COLOR + 44, SetColor),
   EV_COMMAND_AND_ID(COLOR + 45, SetColor),
   EV_COMMAND_AND_ID(COLOR + 46, SetColor),
   EV_COMMAND_AND_ID(COLOR + 47, SetColor),
   EV_COMMAND_AND_ID(COLOR + 48, SetColor),
   EV_COMMAND_AND_ID(COLOR + 49, SetColor),
   EV_COMMAND_AND_ID(COLOR + 50, SetColor),
   EV_COMMAND_AND_ID(COLOR + 52, SetColor),
   EV_COMMAND_AND_ID(COLOR + 53, SetColor),
   EV_COMMAND_AND_ID(COLOR + 54, SetColor),
   EV_COMMAND_AND_ID(COLOR + 55, SetColor),
   EV_COMMAND_AND_ID(COLOR + 56, SetColor),
   EV_COMMAND_AND_ID(COLOR + 57, SetColor),
   EV_COMMAND_AND_ID(COLOR + 58, SetColor),
   EV_COMMAND_AND_ID(COLOR + 59, SetColor),
   EV_COMMAND_AND_ID(COLOR + 60, SetColor),
   EV_COMMAND_AND_ID(COLOR + 62, SetColor),
   EV_COMMAND_AND_ID(COLOR + 63, SetColor),
   EV_COMMAND_AND_ID(COLOR + 64, SetColor),
   EV_COMMAND_AND_ID(COLOR + 65, SetColor),
   EV_COMMAND_AND_ID(COLOR + 66, SetColor),
   EV_COMMAND_AND_ID(COLOR + 67, SetColor),
   EV_COMMAND_AND_ID(COLOR + 68, SetColor),
   EV_COMMAND_AND_ID(COLOR + 69, SetColor),
   EV_COMMAND_AND_ID(COLOR + 70, SetColor),
   EV_COMMAND_AND_ID(COLOR + 72, SetColor),
   EV_COMMAND_AND_ID(COLOR + 73, SetColor),
   EV_COMMAND_AND_ID(COLOR + 74, SetColor),
   EV_COMMAND_AND_ID(COLOR + 75, SetColor),
   EV_COMMAND_AND_ID(COLOR + 76, SetColor),
   EV_COMMAND_AND_ID(COLOR + 77, SetColor),
   EV_COMMAND_AND_ID(COLOR + 78, SetColor),
   EV_COMMAND_AND_ID(COLOR + 79, SetColor),
   EV_COMMAND_AND_ID(COLOR + 80, SetColor),
   EV_COMMAND_AND_ID(COLOR + 82, SetColor),
   EV_COMMAND_AND_ID(COLOR + 83, SetColor),
   EV_COMMAND_AND_ID(COLOR + 84, SetColor),
   EV_COMMAND_AND_ID(COLOR + 85, SetColor),
   EV_COMMAND_AND_ID(COLOR + 86, SetColor),
   EV_COMMAND_AND_ID(COLOR + 87, SetColor),
   EV_COMMAND_AND_ID(COLOR + 88, SetColor),
   EV_COMMAND_AND_ID(COLOR + 89, SetColor),
   EV_COMMAND_AND_ID(COLOR + 90, SetColor),
   EV_COMMAND_AND_ID(COLOR + 92, SetColor),
   EV_COMMAND_AND_ID(COLOR + 93, SetColor),
   EV_COMMAND_AND_ID(COLOR + 94, SetColor),
   EV_COMMAND_AND_ID(COLOR + 95, SetColor),
   EV_COMMAND_AND_ID(COLOR + 96, SetColor),
   EV_COMMAND_AND_ID(COLOR + 97, SetColor),
   EV_COMMAND_AND_ID(COLOR + 98, SetColor),
   EV_COMMAND_AND_ID(COLOR + 99, SetColor),
   EV_COMMAND_AND_ID(COLOR + 100, SetColor),
   EV_COMMAND_AND_ID(COLOR + 102, SetColor),
   EV_COMMAND_AND_ID(COLOR + 103, SetColor),
   EV_COMMAND_AND_ID(COLOR + 104, SetColor),
   EV_COMMAND_AND_ID(COLOR + 105, SetColor),
   EV_COMMAND_AND_ID(COLOR + 106, SetColor),
   EV_COMMAND_AND_ID(COLOR + 107, SetColor),
   EV_COMMAND_AND_ID(COLOR + 108, SetColor),
   EV_COMMAND_AND_ID(COLOR + 109, SetColor),

   EV_COMMAND_AND_ID(THICKNESS, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 2, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 3, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 4, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 5, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 6, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 7, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 8, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 9, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 10, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 12, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 13, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 14, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 15, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 16, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 17, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 18, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 19, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 20, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 22, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 23, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 24, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 25, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 26, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 27, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 28, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 29, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 30, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 32, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 33, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 34, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 35, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 36, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 37, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 38, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 39, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 40, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 42, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 43, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 44, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 45, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 46, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 47, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 48, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 49, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 50, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 52, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 53, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 54, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 55, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 56, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 57, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 58, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 59, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 60, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 62, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 63, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 64, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 65, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 66, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 67, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 68, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 69, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 70, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 72, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 73, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 74, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 75, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 76, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 77, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 78, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 79, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 80, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 82, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 83, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 84, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 85, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 86, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 87, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 88, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 89, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 90, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 92, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 93, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 94, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 95, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 96, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 97, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 98, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 99, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 100, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 102, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 103, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 104, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 105, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 106, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 107, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 108, SetThick),
   EV_COMMAND_AND_ID(THICKNESS + 109, SetThick),
END_RESPONSE_TABLE;



TGraphWindow::TGraphWindow(TFrameWindow* frame, TWindow* parent, char* titl, int nyg,
      					int* ny, char* xName, char** ygNames, char*** yNames,
                     char* xUnits, char** yUnits, TArray<double>* x,
                     TArray<double>*** y, bool delArrays = false)
                  	: TWindow(parent, titl)

	// if DelArrays is set to true, TGraphWindow will delete the TArrays
   // nyg indicates the number of y groups.
   // ny indicates the size of a given group.
{
	DelArrays = delArrays;
	Nyg = nyg;
   Ny = ny;
   XName = xName;
   YGNames = ygNames;      // names of groups
   YNames = yNames;			// names of individual plot items
   XUnits = xUnits;
   YUnits = yUnits;
   title = new char[80];
   strcpy (title, titl);
   X = x;
   Y = y;
   DisplayYG = new bool[Nyg];
   DisplayY = new bool*[Nyg];
   for (int i = 0; i < Nyg; i++) {
   	DisplayY[i] = new bool[Ny[i]];
   }
   DisplayTitle = true;
   for (int i = 0; i < Nyg; i++) {
   	DisplayYG[i] = (i < 5);              		// 5 groups are displayed initially
      for (int j = 0; j < Ny[i]; j++) {
      	DisplayY[i][j] = true;
      }
   }

   YGColor = new TColor*[Nyg];
   YColor = new TColor**[Nyg];
   for (int i = 0; i < Nyg; i++) {
   	switch (i%5) {
      	case 0 : YGColor[i] = new TColor(DEF_COLOR0);
         			break;
         case 1 : YGColor[i] = new TColor(DEF_COLOR1);
         			break;
         case 2 : YGColor[i] = new TColor(DEF_COLOR2);
         			break;
         case 3 : YGColor[i] = new TColor(DEF_COLOR3);
         			break;
         case 4 : YGColor[i] = new TColor(DEF_COLOR4);
      }
   	YColor[i] = new TColor*[Ny[i]];
      for (int j = 0; j < Ny[i]; j++) {
      	YColor[i][j] = new TColor(YGColor[i]->GetValue());
      }
   }

   Thickness = new int*[Nyg];
   for (int i = 0; i < Nyg; i++) {
      Thickness[i] = new int[Ny[i]];
      for (int j = 0; j < Ny[i]; j++) {
      	Thickness[i][j] = DEF_THICKNESS;
      }
  }

	DefPen = new TPen(TColor::Black, 1, PS_SOLID);

	Frame = frame;
   Frame->AssignMenu(GRAPH_MENU);
   Printer = new TPrinter;
}


TGraphWindow::~TGraphWindow()
{
	if (DelArrays) {  // this says we should delete the TArrays ourselves
   	delete X;
      for (int i = 0; i < Nyg; i++) {
      	for (int j = 0; j < Ny[i]; j++) {
		     	delete Y[i][j];
            delete [] YNames[i][j];
         }
         delete Y[i];
         delete YNames[i];
         delete [] YGNames[i];
         delete [] YUnits[i];
      }
      delete [] Y;
      delete [] YNames;
      delete [] YGNames;
      delete [] YUnits;
      delete [] XName;
      delete [] XUnits;
   }
   for (int i = 0; i < Nyg; i++) {
   	for (int j = 0; j < Ny[i]; j++) {
         delete YColor[i][j];
      }
      delete DisplayY[i];
   	delete YColor[i];
      delete YGColor[i];
      delete [] Thickness[i];
   }
   delete [] DisplayY;
	delete [] DisplayYG;
   delete YColor;
   delete YGColor;
   delete Thickness;
   delete DefPen;
   delete Printer;
   delete title;
}


void
TGraphWindow::SetupWindow()
{
	TWindow::SetupWindow();

	TMenu* menu;
	menu = new TMenu(Frame->HWindow);
   SetupMenu(menu);
  	delete menu;
}


void
TGraphWindow::CmCopy()
{
	TClientDC dc(*this);
   TRect rc(GetClientRect());

   TMemoryDC memDC(dc);
   TBitmap bitmap(dc, rc.Width(), rc.Height());
   memDC.SelectObject(bitmap);
   TBrush BlankBrush(memDC.GetBkColor());
   memDC.FillRect(rc, BlankBrush);
   Paint(memDC, true, rc);

   TClipboard clipboard(*this, false);
   if (clipboard && clipboard.EmptyClipboard())
   	bitmap.ToClipboard(clipboard);

}


void
TGraphWindow::CmPrint()
{
	if (Printer) {
		TWindowPrintout printout(title, this);
   	printout.SetBanding(true);
   	Printer->Print(this, printout, true);
  	}
}


void
TGraphWindow::CmExit()
{
	if (CanClose()) Destroy();
}

void
TGraphWindow::CmPageSetup()
{
	if (Printer)
   	Printer->Setup(this);
}



void
TGraphWindow::SetupMenu(TMenu* menu)
{
	TPopupMenu* Popup;
	menu->InsertMenu(DISPLAY1, 0, DISPLAY - 3, "Title");
	for (int i = 0; i < Nyg; i++) {
		menu->InsertMenu(DISPLAY1, 0, DISPLAY + i*10, YGNames[i]);
      menu->InsertMenu(COLOR1, 0, COLOR + i*10, YGNames[i]);
      menu->InsertMenu(THICKNESS1, 0, THICKNESS + i*10, YGNames[i]);

		if (Ny[i] > 1) {
	   	Popup = new TPopupMenu();
   	   for (int j = 0; j < Ny[i]; j++) {
			   Popup->AppendMenu(MF_ENABLED, DISPLAY + i*10 + 2 + j, YNames[i][j]);
   	   }
	      ::InsertMenu(menu->GetHandle(), DISPLAY1, MF_BYCOMMAND|MF_POPUP|MF_STRING,
   	    				 UINT(Popup->GetHandle()), YGNames[i]);

	   	Popup = new TPopupMenu();
   	   for (int j = 0; j < Ny[i]; j++) {
			   Popup->AppendMenu(MF_ENABLED, COLOR + i*10 + 2 + j, YNames[i][j]);
   	   }
	      ::InsertMenu(menu->GetHandle(), COLOR1, MF_BYCOMMAND|MF_POPUP|MF_STRING,
   	    				 UINT(Popup->GetHandle()), YGNames[i]);

	   	Popup = new TPopupMenu();
   	   for (int j = 0; j < Ny[i]; j++) {
			   Popup->AppendMenu(MF_ENABLED, THICKNESS + i*10 + 2 + j, YNames[i][j]);
   	   }
	      ::InsertMenu(menu->GetHandle(), THICKNESS1, MF_BYCOMMAND|MF_POPUP|MF_STRING,
   	    				 UINT(Popup->GetHandle()), YGNames[i]);
      }
   }
	menu->DeleteMenu(DISPLAY1, MF_BYCOMMAND);
   menu->DeleteMenu(COLOR1, MF_BYCOMMAND);
   menu->DeleteMenu(THICKNESS1, MF_BYCOMMAND);   // these were place holders
}


void
TGraphWindow::CommandEnabler (TCommandEnabler& tce)
{
	TCommandEnabler::TCommandState checkstate;
   if (tce.Id >= DISPLAY) {
   	int diff = tce.Id - DISPLAY;
      if (diff / 10 > Nyg) return;   // this item is not in the menu
      if (diff % 10 == 0) {
	   	checkstate = (DisplayYG[diff/10]) ? TCommandEnabler::Checked :
         														TCommandEnabler::Unchecked;
      }
      else {
      	if (diff % 10 - 2 > Ny[diff/10]) return;   // this item is not in the menu
      	checkstate = DisplayY[diff/10][diff%10 - 2] ? TCommandEnabler::Checked :
         														TCommandEnabler::Unchecked;
      }
		tce.SetCheck(checkstate);
   		// check it if it is to be displayed
   }
   else {
   	checkstate = (DisplayTitle) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked;
   	tce.SetCheck(checkstate);
   }
}


void
TGraphWindow::InvDisplay (WPARAM ID)
{
	ID -= DISPLAY;
   int i = ID / 10;
   int j = ID % 10 - 2;
	if (j < 0) DisplayYG[i] = !DisplayYG[i];
   else DisplayY[i][j] = !DisplayY[i][j];
   Invalidate();
}

void
TGraphWindow::DispTitle ()
{
	DisplayTitle = !DisplayTitle;
   Invalidate();
}


void
TGraphWindow::SetColor(WPARAM ID)
{
	ID -= COLOR;
   int i = ID / 10;
   int j = ID % 10 - 2;

	TChooseColorDialog::TData colors;
   static TColor custColors[16] =
   {
   	0x010101L, 0x101010L, 0x202020L, 0x303030L,
      0x404040L, 0x505050L, 0x606060L, 0x707070L,
      0x808080L, 0x909090L, 0xA0A0A0L, 0xB0B0B0L,
      0xC0C0C0L, 0xD0D0D0L, 0xE0E0E0L, 0xF0F0F0L
   };

   colors.Flags = CC_RGBINIT;
   char title[80];
   if (j < 0) {  // then a group was selected
   	colors.Color = *YGColor[i];
      strcpy(title, YGNames[i]);
   }
   else {
   	colors.Color = *YColor[i][j];
      strcpy(title, YNames[i][j]);
   }
   colors.CustColors = custColors;
   if (TChooseColorDialog(this, colors, 0, title).Execute() == IDOK) {
   	if (j < 0) {
	      *YGColor[i] = colors.Color;
   	   for (int k = 0; k < Ny[i]; k++) {
      		*YColor[i][k] = colors.Color;
	      }
      }
      else {
      	*YGColor[i] = TColor::Black;   // set axis color to black
         *YColor[i][j] = colors.Color;
      }
      Invalidate();
   }
}

void
TGraphWindow::SetThick(WPARAM ID)
{
	ID -= THICKNESS;
   int i = ID / 10;
   int j = ID % 10 - 2;
   char title[80];
   int* CurrThick;
   if (j < 0) {   // then a group was selected
   	CurrThick = &Thickness[i][0];
      strcpy(title, YGNames[i]);
   }
   else {
   	CurrThick = &Thickness[i][j];
   	strcpy(title, YNames[i][j]);
   }
   if (TThickDialog(this, title
   , *CurrThick).Execute() == IDOK) {
   	if (j < 0) {	// a group was selected
	   	for (int k = 1; k < Ny[i]; k++) {
   	 		Thickness[i][k] = Thickness[i][0];
   		}
      }
   	Invalidate();
	}
}


void
TGraphWindow::EvSize(UINT sizeType, TSize& size)
{
  TWindow::EvSize(sizeType, size);
  Invalidate();

  if (sizeType == SIZE_MINIMIZED) {
  		Paint(TClientDC(*Parent), FALSE, TRect());
  }
}


void
TGraphWindow::Paint(TDC& dc, bool, TRect&)
{
	TRect WinRect, GRect;   // window size and graph size
  	GetClientRect(WinRect);


 /*	// We speed things up by writing to a memory DC:

   TMemoryDC  memDC(dc);

   TBitmap bitmap(dc, WinRect.Width(), WinRect.Height());
   memDC.SelectObject(bitmap);

   // Initiallize the new bitmap to all white.
   //
   memDC.PatBlt(WinRect, WHITENESS);

 */


   int TopSpace = TOP_SPACE;

   if (DisplayTitle) {
   	TFont TitleFont(GR_FONT, TITLE_FONT_HEIGHT, 0, 0, 0, FW_BOLD);
      dc.SelectObject(TitleFont);
      TSize size = dc.GetTextExtent(title, strlen(title));
      TPoint pos((WinRect.right - size.cx)/2, TOP_SPACE);
      dc.TextOut(pos, title);
      dc.RestoreFont();
      TopSpace += size.cy;
   }

// set up default font:
   TFont font(GR_FONT, GR_FONT_HEIGHT);
   dc.SelectObject(font);

   // first figure out top and bottom of graph
   GRect.top = WinRect.top + TopSpace;
   TSize XLSize = dc.GetTextExtent(XName, strlen(XName));
   int BottomSpace = BOTTOM_SPACE + 2*XLSize.cy + 4 + XAXIS_TITLE_SPACE
   																			+ XTIC_SPACE;
   GRect.bottom = WinRect.bottom - BottomSpace;

// now draw Y axes:
	int LeftSpace = YAxes(dc, GRect);

// figure out left and right of graph

	int RightSpace = RIGHT_SPACE;

   GRect.right = WinRect.right - RightSpace;
   GRect.left = WinRect.left + LeftSpace;

   dc.SelectObject(font);
// now draw X axis:
   XAxis(dc, GRect, WinRect);

   TPoint GSize(GRect.right - GRect.left, GRect.bottom - GRect.top);
	DrawGraph(dc, GRect, GSize);

/*
   // Now copy the memDC to the screen DC:
   TPoint origin(0, 0);
   dc.BitBlt(WinRect, memDC, origin, SRCCOPY);

   // Restore original bitmap before leaving
   //
   memDC.RestoreBitmap();
*/
}


int
TGraphWindow::YAxes(TDC& dc, TRect& GRect)
{
	int VSize = GRect.bottom - GRect.top;
   double Scale;      // scale of a given axis
   double YNum;    // y interval for tics
   int TicOffset;  // offset for tics  (i.e. width of largest # label)
   char buf[30];    // buffer for holding text for gcvt
   char label1[30];  // y axis label, part 1
   char label2[5];   // part 2; the superscripted part
   char label3[30];  // part 3; the unit
   double YVal;
   int width;      // width of current label
   double factor;     // multiplicitive factor to put on axis
   double null;
   int exponent;      // exponent of multiplicitive factor
   TSize YLSize1, YLSize2, YLSize3, YLSize;

   TPoint LPos(0, 0);  // position of label
   TPoint RPos;
   TFont font(GR_FONT, NUM_FONT_HEIGHT);
   TFont GrFont(GR_FONT, GR_FONT_HEIGHT);
   dc.SelectObject(GrFont);
   Text(dc, LPos, "", true, true);
   bool dontDraw;
	for (int i = 0; i < Nyg; i++) {
      if (!DisplayYG[i]) continue;   // exit loop if we should not display Y
      dontDraw = true;
      for (int j = 0; j < Ny[i]; j++) {
			if (DisplayY[i][j]) dontDraw = false;
      }
      if (dontDraw) continue; // exit loop if no items are selected

      dc.SetTextColor(*YGColor[i]);
      // first check if the labels are > 2000 or < 0.1:
      exponent = 0;  // defaults
      factor = 1.0;
      YNum = GetAxisLabelSize(MaxY(i));   // get step size for number labels on axis
		if ((YNum > 2000)||(YNum < 0.1)) {
      	split(YNum, null, exponent);
         factor = splice(1.0, exponent);
      }

		// write name of label:
      strcpy(label1, YGNames[i]);
      strcat(label1, ",");
      strcpy(label2, "");   // default for label2
      if (exponent) {
      	strcat(label1, " 10");
         itoa(exponent, buf, 10);
         strcpy(label2, buf);
      }
      strcpy(label3, " ");
      strcat(label3, YUnits[i]);
   	YLSize1 = dc.GetTextExtent(label1, strlen(label1));
		YLSize2 = dc.GetTextExtent(label2, strlen(label2));
      YLSize3 = dc.GetTextExtent(label3, strlen(label3));
		YLSize.cx = YLSize1.cx + YLSize2.cx + YLSize3.cx;
      YLSize.cy = YLSize1.cy;
   	LPos.x += LEFT_SPACE;
   	LPos.y = (VSize + YLSize.cx)/2 + GRect.top;
   	Text(dc, LPos, label1, true, true);
      LPos.y = LPos.y - YLSize1.cx;
      LPos.x -= YLSize.cy/2;  // move up 1/2 a character size
      Text(dc, LPos, label2, true, true);
      LPos.y = LPos.y - YLSize2.cx;
      LPos.x += YLSize.cy/2;
      Text(dc, LPos, label3, true, true);

      // now label axis with #'s:
      LPos.x += YLSize.cy + YAXIS_TITLE_SPACE;
              // we use the y size of the text because it is sideways

  		// get Y Scale:
   	Scale = GetYScale(VSize, i);

      // Now we go through all numbers on axis 2x; the first time, we just
      // find the maximum length

      TicOffset = 0;

      for (int j = 0; j < 10; j++) {
      	YVal = YNum * double(j);
      	LPos.y = GRect.bottom - int(Scale * YVal);
         if (LPos.y < GRect.top) continue;  // i.e. don't go beyond the size
         											   // of the graph
			gcvt((YVal/factor), digits(YVal/factor), buf);
         width = (dc.GetTextExtent(buf, strlen(buf))).cx;
         if (width > TicOffset) TicOffset = width;
         		// this sets the TicOffset to the size of the largest label
      }
		LPos.x += TicOffset;

      // now we actually do the writing:
     	dc.SelectObject(font);
      for (int j = 0; j < 10; j++) {
      	YVal = YNum * double(j);
      	LPos.y = GRect.bottom - int(Scale * YVal);
         if (LPos.y < GRect.top) continue;  // i.e. don't go beyond the size
         											   // of the graph
     			// this subtraction serves to center the # at the correct position
			gcvt((YVal/factor), digits(YVal/factor), buf);
         dc.SetTextAlign(TA_RIGHT);
         LPos.y -= dc.GetTextExtent(buf, strlen(buf)).cy/2;
         Text(dc, LPos, buf, false, false, NUM_FONT_HEIGHT);
      }
      dc.SetTextAlign(TA_LEFT);  // go back to normal

      // put in axis tics
      LPos.x += YTIC_SPACE;
		for (int j = 0; j < 10; j++) {
     		YVal = YNum * double(j);
      	LPos.y = GRect.bottom - int(Scale * YVal);
         if (LPos.y < GRect.top) continue;  // i.e. don't go beyond the size
         											  // of the graph
			dc.MoveTo(LPos);
         RPos.x = LPos.x + TIC_LEN;
         RPos.y = LPos.y;
         MyLineTo(dc, RPos);
      }
      LPos.x += TIC_LEN;
      // finally, draw a vertical axis:
      LPos.y = GRect.top;
      dc.MoveTo(LPos);
      LPos.y = GRect.bottom;
      MyLineTo(dc, LPos);
   }
   dc.SetTextColor(TColor::Black);
   return LPos.x;  // this is how much space is taken up
}


void
TGraphWindow::XAxis(TDC& dc, TRect& GRect, TRect& WRect)
{
	int HSize = GRect.right - GRect.left;
   double Scale;      // scale of a given axis
   double XNum;    	 // x interval for tics
   char buf[30];    	 // buffer for holding text for gcvt
   char label1[30];   // x axis label, part 1
   char label2[5];    // part 2; the superscripted part
   char label3[30];   // part 3; the unit
   double XVal;
   double factor;     // multiplicitive factor to put on axis
   double null;
   int exponent;      // exponent of multiplicitive factor
   TSize XLSize1, XLSize2, XLSize3, XLSize;

   TPoint LPos(0, 0);  // position of label
   TPoint RPos;

    // first check if the labels are > 2000 or < 0.1:
   exponent = 0;  // defaults
   factor = 1.0;
	XNum = GetAxisLabelSize(MaxX());   // get step size for number labels on axis
	if ((XNum > 2000)||(XNum < 0.1)) {
     	split(XNum, null, exponent);
      factor = splice(1.0, exponent);
   }

	// write name of label:
   strcpy(label1, XName);
   strcat(label1, ",");
   strcpy(label2, "");   // default for label2
   if (exponent) {
    	strcat(label1, " 10");
      itoa(exponent, buf, 10);
      strcpy(label2, buf);
   }
   strcpy(label3, " ");
   strcat(label3, XUnits);
	XLSize1 = dc.GetTextExtent(label1, strlen(label1));
	XLSize2 = dc.GetTextExtent(label2, strlen(label2));
   XLSize3 = dc.GetTextExtent(label3, strlen(label3));
	XLSize.cx = XLSize1.cx + XLSize2.cx + XLSize3.cx;
   XLSize.cy = XLSize1.cy;
   LPos.x = (HSize - XLSize.cx)/2 + GRect.left;
   LPos.y = WRect.bottom - BOTTOM_SPACE - XLSize.cy;
   Text(dc, LPos, label1, true, false, X_FONT_HEIGHT);
   LPos.x += XLSize1.cx + 4;
   LPos.y -= XLSize.cy/2;  // move up 1/2 a character size
   Text(dc, LPos, label2, true, false, X_FONT_HEIGHT);
   LPos.x += XLSize2.cx;
   LPos.y += XLSize.cy/2;
   Text(dc, LPos, label3, true, false, X_FONT_HEIGHT);

   // now label axis with #'s:
   LPos.y += - XLSize.cy - XAXIS_TITLE_SPACE;
  		// get X Scale:
 	Scale = GetXScale(HSize);

   // now we actually do the writing of the #'s:

   for (int j = 0; j < 10; j++) {
    	XVal = XNum * double(j);
     	LPos.x = GRect.left + int(Scale * XVal);
      if (LPos.x > GRect.right) continue;  // i.e. don't go beyond the size
         											   // of the graph
		gcvt((XVal/factor), digits(XVal/factor), buf);
		// this subtraction serves to center the # at the correct position:
      LPos.x += -dc.GetTextExtent(buf, strlen(buf)).cx / 2 + 2;
      Text(dc, LPos, buf, false, false, NUM_FONT_HEIGHT);
   }

   // put in axis tics
   LPos.y -= XTIC_SPACE;
	for (int j = 0; j < 10; j++) {
  		XVal = XNum * double(j);
     	LPos.x = GRect.left + int(Scale * XVal);
      if (LPos.x > GRect.right) continue;  // i.e. don't go beyond the size
         						   				 // of the graph
		dc.MoveTo(LPos);
      RPos.x = LPos.x;
      RPos.y = LPos.y - TIC_LEN;
      MyLineTo(dc, RPos);
   }
   LPos.y -= TIC_LEN;
}




void
TGraphWindow::DrawGraph(TDC& dc, TRect& GRect, TPoint& GSize)
{
   dc.Rectangle(GRect);
   Plot(dc, GRect, GSize);  // plot all the desired y series
}


void
TGraphWindow::Plot(TDC& dc, TRect& GRect, TPoint& GSize)
{
	double XScale, YScale;
   double x;
   double x1 = 0;
   double y1 = 0;
	for (int i = 0; i < Nyg; i++) {      // step through each y group
   	if (!DisplayYG[i]) continue;      // skip if we're not supposed to display
      for (int j = 0; j < Ny[i]; j++) {
      	if (!DisplayY[i][j]) continue;
	      TPen YPen(*YColor[i][j], Thickness[i][j]);
   	   dc.SelectObject(YPen);
      	YScale = GetYScale(GSize.y, i);
	      XScale = GetXScale(GSize.x);
			// now we step through each x pixel (j is the pixel #):
        	TArrayIterator<double> iter(*Y[i][j]);
        	TArrayIterator<double> iterX(*X);
	   	dc.MoveTo(GRect.left, GRect.bottom - YScale*GetY(0, iterX, iter, x1, y1));
		   for (int k = (Thickness[i][j]+ 3)/2; k <= GSize.x;	k += Thickness[i][j]) {
   			x = double(k) / XScale;
      		dc.LineTo(GRect.left + k, GRect.bottom - int(YScale*GetY(x, iterX, iter, x1, y1)));
		   }
      }
   }
   dc.SelectObject(*DefPen);
}


double
TGraphWindow::GetAxisLabelSize(double maxval)
{
	double val;
   val = maxval / DIVISIONS;   // we want approx DIVISIONS labels
	val = getnearest(val);
   return val;
}


double
TGraphWindow::GetXScale(int XSize)
{
   return double(XSize) / MaxX();
}

double
TGraphWindow::GetYScale(int YSize, int i)
{
	return double(YSize) / MaxY(i);
}


double
TGraphWindow::MaxX()
{
	return Max(X);
}

double
TGraphWindow::MaxY(int i)
{
	double val;
	double MaxVal = -1E99;
   for (int j = 0; j < Ny[i]; j++) {
		val = Max(Y[i][j]);
      if (val > MaxVal) MaxVal = val;
   }
	return MaxVal;
}


double
TGraphWindow::Max(TArray<double>* V)
{
   TArrayIterator<double> iter(*V);
   double MaxVal = -1E99;
   while (int(iter))  {  // i.e. while there is something left in the iterator:
		if (iter.Current() > MaxVal) MaxVal = iter.Current();
      iter++;
   }
   return MaxVal;
}



double
TGraphWindow::GetY (double x, TArrayIterator<double>& iterX, TArrayIterator<double>& iter,
							double& x1, double& y1)
{
	double x2, y2;
 	while (x >= iterX.Current() && int(iterX)) {
		x1 = iterX++;
		y1 = iter++;
	}
	x2 = iterX.Current();
	y2 = iter.Current();
	if (abs(x-x1) < 0.0001*(x2-x1)) return y1;
	if (abs(x-x2) < 0.0001*(x2-x1)) return y2;
	return y1 + (x-x1)*(y2 - y1)/(x2-x1);   // linear interpolation
}



void
TGraphWindow::Text(TDC& dc, TPoint point, char* text, bool bold, bool vertical,
							int size)
{
   TFont font(GR_FONT, size, 0, vertical ? 900 : 0, 0, bold ? FW_BOLD : FW_NORMAL);
   dc.SelectObject(font);
   dc.TextOut(point, text, lstrlen(text));
   dc.RestoreFont();  // go back to the original font
}



bool
TGraphWindow::MyLineTo(TDC& dc, TPoint& point)
{
	return dc.LineTo(point.x, point.y) && dc.LineTo(point.x + 1, point.y);
}

bool
TGraphWindow::MyLineTo(TDC& dc, int x, int y)
{
	return dc.LineTo(x, y) && dc.LineTo(x + 1, y);
}


//----------------------------------------------------------



