//=========================================================================
//  TGraphWindow 1.0
//
//  M.E. Burkard    Copyright (C) 1998
//
//  OWL 5.0 / BC++ 5.01A code
//  A TWindow-derived class which draws a graph based on data passed.
//
//  THIS PROGRAM MAY BE FREELY USED AND MODIFIED
//  Mark Burkard is not liable from any results of the use or misuse
//  of this program.
//=========================================================================
//
//  Header file
//
//  M.E. Burkard    3/97
//=========================================================================

#if !defined(TGRAPHWINDOW_H)
#define TGRAPHWINDOW_H

#include <owl\printer.h>

#define RIGHT_SPACE 10         // these specify where how large the graph
#define TOP_SPACE 10  			 // is in the window; in pixels
#define BOTTOM_SPACE 10
#define LEFT_SPACE 12
#define YAXIS_TITLE_SPACE 4
#define XAXIS_TITLE_SPACE 4
#define TIC_LEN 5;
#define YTIC_SPACE 2;
#define XTIC_SPACE 2;
#define DIVISIONS 5;          // the approx. # of labels per axis


#define GR_FONT "Times New Roman"
#define TITLE_FONT_HEIGHT 25
#define GR_FONT_HEIGHT 16
#define X_FONT_HEIGHT 18
#define NUM_FONT_HEIGHT 14


// defines for resource menu:
#define FILE 9060
#define EDIT 9062
#define PRINT_SETUP 9070
#define PAGE_SETUP 9071
#define PRINT 9072
#define EXIT 9073
#define COPY 9074
#define GRAPH_MENU 9096
#define DISPLAY1 9099
#define COLOR1 9299
#define DISPLAY 9100
#define COLOR 9300
#define THICKNESS 9500
#define THICKNESS1 9499
#define DEF_COLOR0 TColor::LtRed
#define DEF_COLOR1 TColor::LtBlue
#define DEF_COLOR2 TColor::LtGreen
#define DEF_COLOR3 TColor::Black
#define DEF_COLOR4 TColor::LtYellow

#define DEF_THICKNESS 1


class TGraphWindow : public TWindow
{
	public:
		TGraphWindow(TFrameWindow* frame, TWindow* parent, char* titl, int nyg,
      					int* ny, char* xName, char** ygNames, char*** yNames,
                     char* xUnits, char** yUnits, TArray<double>* x,
                     TArray<double>*** y, bool delArrays);
		~TGraphWindow();

   protected:
   	void SetupWindow();
   	void Paint(TDC& dc, bool clear, TRect&);
      void EvSize(UINT sizeType, TSize& size);
      void Text(TDC& dc, TPoint point, char* text, bool bold = false,
      							bool vertical = false, int size = GR_FONT_HEIGHT);
      double GetY (double x, TArrayIterator<double>& iterX, TArrayIterator<double>& iter,
							double& x1, double& y1);
      void DrawGraph(TDC& dc, TRect& GRect, TPoint& GSize);
      void Plot(TDC& dc, TRect& GRect, TPoint& GSize);
      double GetXScale(int XSize);
      double GetYScale(int YSize, int i);
      double MaxX();
      double MaxY(int i);
      double Max(TArray<double>* V);
      double GetAxisLabelSize(double maxval);
      int YAxes(TDC& dc, TRect& GRect);    // returns the amount of space taken up
      void XAxis(TDC& dc, TRect& GRect, TRect& WRect);
      bool MyLineTo(TDC& dc, TPoint& point);
      bool MyLineTo(TDC& dc, int x, int y);
      void SetupMenu(TMenu* menu);
      void CommandEnabler (TCommandEnabler& tce);
      void InvDisplay(WPARAM ID);
      void SetColor(WPARAM ID);
      void SetThick(WPARAM ID);
      void CmPageSetup();
      void CmExit();
      void CmPrint();
      void CmCopy();

      void DispTitle();
      void Thick0()     {SetThick(0);}
      void Thick1()     {SetThick(1);}
      void Thick2()     {SetThick(2);}
      void Thick3()     {SetThick(3);}
      void Thick4()     {SetThick(4);}
      void Thick5()     {SetThick(5);}
      void Thick6()     {SetThick(6);}
      void Thick7()     {SetThick(7);}
      void Thick8()     {SetThick(8);}
      void Thick9()     {SetThick(9);}
      void Thick10()    {SetThick(10);}
      void Thick11()    {SetThick(11);}
      void Thick12()    {SetThick(12);}
      void Thick13()    {SetThick(13);}
      void Thick14()    {SetThick(14);}

   private:
   	int Nyg;
      int* Ny;
      char* XName;
      char** YGNames;
      char*** YNames;
      char* XUnits;
      char** YUnits;
      TArray<double>* X;
      TArray<double>*** Y;
      bool* DisplayYG;  // should we show a group?
      bool** DisplayY;  // should we show a particular group member?
      bool DisplayTitle;
      TFrameWindow* Frame;
      TColor** YGColor;         // color of group
      TColor*** YColor;
      TPen* DefPen;
      int** Thickness;
      TPrinter* Printer;
      char* title;
      bool DelArrays;
 
   DECLARE_RESPONSE_TABLE(TGraphWindow);
};


#endif   //defined TGRAPHWINDOW_H
